//************************************************************************
// Example2GCD.java
//
// Presents and tests a few algorithms for finding the greatest common
// divisor of two integers.
//************************************************************************

class Example2GCD
{
   //=====================================================================
   //  Examines several methods for finding greatest common divisor.
   //=====================================================================
   public static void main (String[] args)
   {
      int m = (int) (100 * Math.random());
      int n = (int) (100 * Math.random());
      if (m>0 || n >0)
         System.out.println ("The first number is " + m +
                             "\nThe second number is " + n +
                             "\nMethod 1 gives: gcd = " + GCD.find_1(m,n) +
                             "\nMethod 2 gives: gcd = " + GCD.find_2(m,n) +
                             "\nMethod 3 gives: gcd = " + GCD.find_3(m,n) +
                             "\nMethod 4 gives: gcd = " + GCD.find_4(m,n));
      else 
         System.out.println ("Undefined if both are 0");

      // Compare runtimes of methods. Note that methods 3 and 4 are
      // run with quite larger integers.
      long time;
      final int NUMBER_OF_TRIALS = 50000;
      System.out.println ("\nRuntimes for " + NUMBER_OF_TRIALS +
                          " pairs of integers are:");

      // Test method 1
      time = System.currentTimeMillis();
      for (int i=1; i <= NUMBER_OF_TRIALS; i++)
      { 
         m = (int) (Short.MAX_VALUE * Math.random());
         n = (int) (Short.MAX_VALUE * Math.random());
         GCD.find_1(m,n);
      }
      System.out.println ("By method 1: " +
                          (System.currentTimeMillis() - time));

      // Test method 2 
      time = System.currentTimeMillis();
      for (int i=1; i <= NUMBER_OF_TRIALS; i++)
      {
         m = (int) (Short.MAX_VALUE * Math.random());
         n = (int) (Short.MAX_VALUE * Math.random());
         GCD.find_2(m,n);
      }
      System.out.println ("By method 2: " +
                          (System.currentTimeMillis() - time));

      // Test method 3 
      time = System.currentTimeMillis();
      for (int i=1; i <= NUMBER_OF_TRIALS; i++)
      {
         m = (int) (Integer.MAX_VALUE * Math.random());
         n = (int) (Integer.MAX_VALUE * Math.random());
         GCD.find_3(m,n);
      }
      System.out.println ("By method 3: " +
                          (System.currentTimeMillis() - time));

      // Test method 4 
      time = System.currentTimeMillis();
      for (int i=1; i <= NUMBER_OF_TRIALS; i++)
      {
         m = (int) (Integer.MAX_VALUE * Math.random());
         n = (int) (Integer.MAX_VALUE * Math.random());
         GCD.find_4(m,n);
      }
      System.out.println ("By method 4: " +
                          (System.currentTimeMillis() - time));
   }  // method main
}// class Example2GCD


class GCD
{
   //=====================================================================
   //  Contains four methods for finding the g.c.d. of two integers.
   //  We assume both integers to be non-negative, at least one positive.
   //=====================================================================
   public static int find_1 (int m, int n)
   {
      // Tries all integers from 1 up to n, and takes largest divisor
      int divisor = 1;
      for (int i=1; i <= m; i++)
         if ((m % i == 0) && (n % i == 0))
            divisor = i;
      return divisor;
   } //  find_1 method

   public static int find_2 (int m, int n)
   {
      // Tries all integers from min(m,n) until a divisor is found
      int candidate = min(m, n);
      while ((candidate > 1) &&
             ((m % candidate != 0) || (n % candidate != 0)))
         candidate--;
      return candidate;
   } //  find_2 method

   public static int find_3 (int m, int n)
   {
      // Replaces the larger integer by the difference between the two
      // until the smaller becomes 0
      while ((m > 0) && (n>0))
         if (m > n)
            m -= n;
         else
            n -= m;
      return m + n;
   } //  find_3 method

   public static int find_4 (int m, int n)
   {
      // Replaces the larger integer by its remainder modulo the
      // smaller until the smaller becomes 0
      while ((m > 0) && (n>0))
         if (m > n)
            m %= n;
         else
            n %= m;
      return m + n;
   } //  find_4 method

   public static int min (int m, int n)
   {
      // Calculates the minimum of two integers
      int r = m;
      if (n < m)
         r = n;
      return r;
   } //  min method
}// class GCD
